@@ -7,11 +7,15 @@
 #include <Rdefines.h>
 #include "000.types.h"
 #include "colCounts_lowlevel.h"
+#include "naming.h"
 
-SEXP colCounts(SEXP x, SEXP dim, SEXP rows, SEXP cols, SEXP value, SEXP what, SEXP naRm, SEXP hasNA) {
+SEXP colCounts(SEXP x, SEXP dim, SEXP rows, SEXP cols, SEXP value, SEXP what, SEXP naRm, SEXP hasNA, SEXP useNames) {
   SEXP ans;
-  int narm, hasna, what2;
+  int narm, hasna, what2, usenames;
   R_xlen_t ii, nrow, ncol;
+  
+  /* Coercion moved down to C */
+  PROTECT(dim = coerceVector(dim, INTSXP));
 
   /* Argument 'x' and 'dim': */
   assertArgMatrix(x, dim, (R_TYPE_LGL | R_TYPE_INT | R_TYPE_REAL), "x");
@@ -38,9 +42,8 @@ SEXP colCounts(SEXP x, SEXP dim, SEXP rows, SEXP cols, SEXP value, SEXP what, SE
 
   /* Argument 'rows' and 'cols': */
   R_xlen_t nrows, ncols;
-  int rowsType, colsType;
-  void *crows = validateIndices(rows, nrow, 0, &nrows, &rowsType);
-  void *ccols = validateIndices(cols, ncol, 0, &ncols, &colsType);
+  R_xlen_t *crows = validateIndices(rows, nrow, 0, &nrows);
+  R_xlen_t *ccols = validateIndices(cols, ncol, 0, &ncols);
 
   /* R allocate an integer vector of length 'ncol' */
   /* R allocate memory for vector 'count' of length 'ncols'.
@@ -48,11 +51,11 @@ SEXP colCounts(SEXP x, SEXP dim, SEXP rows, SEXP cols, SEXP value, SEXP what, SE
   double *count = (double *) R_alloc(ncols, sizeof(double));
 
   if (isReal(x)) {
-    colCounts_dbl[rowsType][colsType](REAL(x), nrow, ncol, crows, nrows, ccols, ncols, asReal(value), what2, narm, hasna, count);
+    colCounts_dbl(REAL(x), nrow, ncol, crows, nrows, ccols, ncols, asReal(value), what2, narm, hasna, count);
   } else if (isInteger(x)) {
-    colCounts_int[rowsType][colsType](INTEGER(x), nrow, ncol, crows, nrows, ccols, ncols, asInteger(value), what2, narm, hasna, count);
+    colCounts_int(INTEGER(x), nrow, ncol, crows, nrows, ccols, ncols, asInteger(value), what2, narm, hasna, count);
   } else if (isLogical(x)) {
-    colCounts_lgl[rowsType][colsType](LOGICAL(x), nrow, ncol, crows, nrows, ccols, ncols, asLogical(value), what2, narm, hasna, count);
+    colCounts_lgl(LOGICAL(x), nrow, ncol, crows, nrows, ccols, ncols, asLogical(value), what2, narm, hasna, count);
   }
 
   /* Coerce counts from double to integer.  This is needed because
@@ -67,7 +70,21 @@ SEXP colCounts(SEXP x, SEXP dim, SEXP rows, SEXP cols, SEXP value, SEXP what, SE
       ans_ptr[ii] = (int)count[ii];
     }
   }
-  UNPROTECT(1);
+  
+  /* Argument 'useNames': */ 
+  usenames = asLogical(useNames);
+  
+  if (usenames != NA_LOGICAL && usenames){
+    SEXP dimnames = getAttrib(x, R_DimNamesSymbol);
+    if (dimnames != R_NilValue) {
+      SEXP namesVec = VECTOR_ELT(dimnames, 1);
+      if (namesVec != R_NilValue) {
+        setNames(ans, namesVec, ncols, ccols);
+      }
+    }
+  }
+  
+  UNPROTECT(2);
 
   return(ans);
 } // colCounts()
@@ -101,16 +118,15 @@ SEXP count(SEXP x, SEXP idxs, SEXP value, SEXP what, SEXP naRm, SEXP hasNA) {
 
   /* Argument 'idxs': */
   R_xlen_t nrows, ncols = 1;
-  int rowsType, colsType = SUBSETTED_ALL;
-  void *crows = validateIndices(idxs, nx, 1, &nrows, &rowsType);
-  void *ccols = NULL;
+  R_xlen_t *crows = validateIndices(idxs, nx, 1, &nrows);
+  R_xlen_t *ccols = NULL;
 
   if (isReal(x)) {
-    colCounts_dbl[rowsType][colsType](REAL(x), nx, 1, crows, nrows, ccols, ncols, asReal(value), what2, narm, hasna, &count);
+    colCounts_dbl(REAL(x), nx, 1, crows, nrows, ccols, ncols, asReal(value), what2, narm, hasna, &count);
   } else if (isInteger(x)) {
-    colCounts_int[rowsType][colsType](INTEGER(x), nx, 1, crows, nrows, ccols, ncols, asInteger(value), what2, narm, hasna, &count);
+    colCounts_int(INTEGER(x), nx, 1, crows, nrows, ccols, ncols, asInteger(value), what2, narm, hasna, &count);
   } else if (isLogical(x)) {
-    colCounts_lgl[rowsType][colsType](LOGICAL(x), nx, 1, crows, nrows, ccols, ncols, asLogical(value), what2, narm, hasna, &count);
+    colCounts_lgl(LOGICAL(x), nx, 1, crows, nrows, ccols, ncols, asLogical(value), what2, narm, hasna, &count);
   }
 
   /* R allocate a scalar */
