@@ -11,6 +11,8 @@ fcns <- list(
 # Subsetted tests
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 source("utils/validateIndicesFramework.R")
+# To check names attribute
+dimnames <- list(letters[1:6], LETTERS[1:6])
 for (fcn in names(fcns)) {
   cat(sprintf("subsetted tests on matrix %s()...\n", fcn))
   row_fcn <- fcns[[fcn]][[1]]
@@ -23,15 +25,22 @@ for (fcn in names(fcns)) {
     storage.mode(w) <- mode
     if (mode == "numeric") w[1] <- Inf
 
-    for (rows in index_cases) {
-      for (cols in index_cases) {
-        for (na.rm in c(TRUE, FALSE)) {
-          validateIndicesTestMatrix_w(x, w, rows, cols,
-                                      ftest = row_fcn, fsure = row_fcn,
-                                      na.rm = na.rm)
-          validateIndicesTestMatrix_w(x, w, rows, cols,
-                                      fcoltest = col_fcn, fsure = row_fcn,
-                                      na.rm = na.rm)
+    # Test with and without dimnames on x
+    for (setDimnames in c(TRUE, FALSE)) {
+      if (setDimnames) dimnames(x) <- dimnames
+      else dimnames(x) <- NULL
+      for (rows in index_cases) {
+        for (cols in index_cases) {
+          for (na.rm in c(TRUE, FALSE)) {
+            for (useNames in c(NA, TRUE, FALSE)) {
+              validateIndicesTestMatrix_w(x, w, rows, cols,
+                                          ftest = row_fcn, fsure = row_fcn,
+                                          na.rm = na.rm, useNames = useNames)
+              validateIndicesTestMatrix_w(x, w, rows, cols,
+                                          fcoltest = col_fcn, fsure = row_fcn,
+                                          na.rm = na.rm, useNames = useNames)
+            }
+          }
         }
       }
     }
